@@ -15,6 +15,7 @@
 
 #include "command_p.h"
 
+#include <utils/applicationstate.h>
 #include <utils/filedialog.h>
 
 #include <Libkleo/Classify>
@@ -178,8 +179,7 @@ bool ExportCertificateCommand::Private::requestFileNames(GpgME::Protocol protoco
         return true;
     }
 
-    KConfigGroup config(KSharedConfig::openConfig(), "ExportDialog");
-    const auto lastDir = config.readEntry("LastDirectory", QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation));
+    const auto lastDir = ApplicationState::lastUsedExportDirectory();
 
     QString proposedFileName = lastDir + QLatin1Char('/');
     if (keys().size() == 1) {
@@ -237,7 +237,7 @@ bool ExportCertificateCommand::Private::requestFileNames(GpgME::Protocol protoco
     }
 
     fileNames[protocol] = fname;
-    config.writeEntry("LastDirectory", fi.absolutePath());
+    ApplicationState::setLastUsedExportDirectory(fi.absolutePath());
     return !fname.isEmpty();
 }
 
@@ -258,8 +258,13 @@ void ExportCertificateCommand::Private::startExportJob(GpgME::Protocol protocol,
         exportResult(result, keyData);
     });
 
-    connect(job.get(), &Job::progress,
+#if QGPGME_JOB_HAS_NEW_PROGRESS_SIGNALS
+    connect(job.get(), &QGpgME::Job::jobProgress,
             q, &Command::progress);
+#else
+    connect(job.get(), &QGpgME::Job::progress,
+            q, [this](const QString &, int current, int total) { Q_EMIT q->progress(current, total); });
+#endif
 
     QStringList fingerprints;
     fingerprints.reserve(keys.size());
