@@ -15,7 +15,7 @@
 
 #include "fileoperationspreferences.h"
 #include <utils/applicationstate.h>
-#ifdef QGPGME_SUPPORTS_SECRET_SUBKEY_EXPORT
+#if QGPGME_SUPPORTS_SECRET_SUBKEY_EXPORT
 #include "utils/filedialog.h"
 #endif
 
@@ -41,7 +41,7 @@ using namespace GpgME;
 namespace
 {
 
-#ifdef QGPGME_SUPPORTS_SECRET_SUBKEY_EXPORT
+#if QGPGME_SUPPORTS_SECRET_SUBKEY_EXPORT
 QString openPGPCertificateFileExtension()
 {
     return QLatin1String{outputFileExtension(Class::OpenPGP | Class::Ascii | Class::Certificate,
@@ -157,7 +157,7 @@ ExportSecretSubkeyCommand::Private::~Private() = default;
 
 void ExportSecretSubkeyCommand::Private::start()
 {
-#ifdef QGPGME_SUPPORTS_SECRET_SUBKEY_EXPORT
+#if QGPGME_SUPPORTS_SECRET_SUBKEY_EXPORT
     if (subkeys.empty()) {
         finished();
         return;
@@ -192,7 +192,7 @@ void ExportSecretSubkeyCommand::Private::cancel()
 
 std::unique_ptr<QGpgME::ExportJob> ExportSecretSubkeyCommand::Private::startExportJob(const std::vector<Subkey> &subkeys)
 {
-#ifdef QGPGME_SUPPORTS_SECRET_SUBKEY_EXPORT
+#if QGPGME_SUPPORTS_SECRET_SUBKEY_EXPORT
     const bool armor = filename.endsWith(u".asc", Qt::CaseInsensitive);
     std::unique_ptr<QGpgME::ExportJob> exportJob{QGpgME::openpgp()->secretSubkeyExportJob(armor)};
     Q_ASSERT(exportJob);
@@ -201,8 +201,13 @@ std::unique_ptr<QGpgME::ExportJob> ExportSecretSubkeyCommand::Private::startExpo
             q, [this](const GpgME::Error &err, const QByteArray &keyData) {
                 onExportJobResult(err, keyData);
             });
-    connect(exportJob.get(), &QGpgME::Job::progress,
+#if QGPGME_JOB_HAS_NEW_PROGRESS_SIGNALS
+    connect(exportJob.get(), &QGpgME::Job::jobProgress,
             q, &Command::progress);
+#else
+    connect(exportJob.get(), &QGpgME::Job::progress,
+            q, [this](const QString &, int current, int total) { Q_EMIT q->progress(current, total); });
+#endif
 
     const GpgME::Error err = exportJob->start(getSubkeyFingerprints(subkeys));
     if (err) {
