@@ -18,6 +18,7 @@
 
 #include "certifycertificatecommand.h"
 #include <utils/keys.h>
+#include <utils/memory-helpers.h>
 #include <settings.h>
 #include "kleopatra_debug.h"
 
@@ -27,6 +28,7 @@
 #include <Libkleo/KeyCache>
 #include <Libkleo/KeyGroupImportExport>
 #include <Libkleo/KeyHelpers>
+#include <Libkleo/MessageBox>
 #include <Libkleo/Predicates>
 #include <Libkleo/Formatting>
 #include <Libkleo/Stl_Util>
@@ -35,7 +37,7 @@
 #include <QGpgME/ImportJob>
 #include <QGpgME/ImportFromKeyserverJob>
 #include <QGpgME/ChangeOwnerTrustJob>
-#ifdef QGPGME_SUPPORTS_RECEIVING_KEYS_BY_KEY_ID
+#if QGPGME_SUPPORTS_RECEIVING_KEYS_BY_KEY_ID
 #include <QGpgME/ReceiveKeysJob>
 #endif
 
@@ -47,7 +49,6 @@
 
 #include <KLocalizedString>
 #include <KMessageBox>
-#include <kwidgetsaddons_version.h>
 
 #include <QByteArray>
 #include <QEventLoop>
@@ -66,6 +67,12 @@ using namespace GpgME;
 using namespace Kleo;
 using namespace QGpgME;
 
+static void disconnectConnection(const QMetaObject::Connection &connection)
+{
+    // trivial function for disconnecting a signal-slot connection
+    QObject::disconnect(connection);
+}
+
 bool operator==(const ImportJobData &lhs, const ImportJobData &rhs)
 {
     return lhs.job == rhs.job;
@@ -377,6 +384,11 @@ static QString make_message_report(const std::vector<ImportResultData> &res,
 // Returns false on error, true if please certify was shown.
 bool ImportCertificatesCommand::Private::showPleaseCertify(const GpgME::Import &imp)
 {
+    if (!Kleo::userHasCertificationKey()) {
+        qCDebug(KLEOPATRA_LOG) << q << __func__ << "No certification key available";
+        return false;
+    }
+
     const char *fpr = imp.fingerprint();
     if (!fpr) {
         // WTF
@@ -386,15 +398,15 @@ bool ImportCertificatesCommand::Private::showPleaseCertify(const GpgME::Import &
     // Exactly one public key imported. Let's see if it is openpgp. We are async here so
     // we can just fetch it.
 
-    auto ctx = GpgME::Context::createForProtocol(GpgME::OpenPGP);
+    auto ctx = wrap_unique(GpgME::Context::createForProtocol(GpgME::OpenPGP));
     if (!ctx) {
         // WTF
         qCWarning(KLEOPATRA_LOG) << "Failed to create OpenPGP proto";
         return false;
     }
+    ctx->addKeyListMode(KeyListMode::WithSecret);
     GpgME::Error err;
-    auto key = ctx->key(fpr, err, false);
-    delete ctx;
+    const auto key = ctx->key(fpr, err, false);
 
     if (key.isNull() || err) {
         // No such key most likely not OpenPGP
@@ -404,6 +416,10 @@ bool ImportCertificatesCommand::Private::showPleaseCertify(const GpgME::Import &
         // key is expired or revoked
         return false;
     }
+    if (key.hasSecret()) {
+        qCDebug(KLEOPATRA_LOG) << q << __func__ << "Secret key is available -> skipping certification";
+        return false;
+    }
 
     for (const auto &uid: key.userIDs()) {
         if (uid.validity() >= GpgME::UserID::Marginal) {
@@ -416,30 +432,17 @@ bool ImportCertificatesCommand::Private::showPleaseCertify(const GpgME::Import &
         << i18n("Using a business card.")
         << i18n("Confirming it on a trusted website.");
 
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
     auto sel = KMessageBox::questionTwoActions(parentWidgetOrView(),
-#else
-    auto sel = KMessageBox::questionYesNo(parentWidgetOrView(),
-#endif
-                                               i18n("In order to mark the certificate as valid (green) it needs to be certified.") + QStringLiteral("<br>")
+                                               i18n("In order to mark the certificate as valid it needs to be certified.") + QStringLiteral("<br>")
                                                    + i18n("Certifying means that you check the Fingerprint.") + QStringLiteral("<br>")
                                                    + i18n("Some suggestions to do this are:")
                                                    + QStringLiteral("<li><ul>%1").arg(suggestions.join(QStringLiteral("</ul><ul>")))
                                                    + QStringLiteral("</ul></li>") + i18n("Do you wish to start this process now?"),
                                                i18nc("@title", "You have imported a new certificate (public key)"),
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
-                                               KGuiItem(i18n("Import")),
+                                               KGuiItem(i18nc("@action:button", "Certify")),
                                                KStandardGuiItem::cancel(),
-#else
-                                          KStandardGuiItem::yes(),
-                                          KStandardGuiItem::no(),
-#endif
                                                QStringLiteral("CertifyQuestion"));
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
     if (sel == KMessageBox::ButtonCode::PrimaryAction) {
-#else
-    if (sel == KMessageBox::Yes) {
-#endif
         QEventLoop loop;
         auto cmd = new Commands::CertifyCertificateCommand(key);
         cmd->setParentWidget(parentWidgetOrView());
@@ -475,6 +478,35 @@ auto getSingleOpenPGPImport(const std::vector<ImportResultData> &res)
     }
     return nullImport;
 }
+
+auto consolidatedAuditLogEntries(const std::vector<ImportResultData> &res)
+{
+    static const QString gpg = QStringLiteral("gpg");
+    static const QString gpgsm = QStringLiteral("gpgsm");
+
+    if (res.size() == 1) {
+        return res.front().auditLog;
+    }
+    QStringList auditLogs;
+    auto extractAndAnnotateAuditLog = [](const ImportResultData &r) {
+        QString s;
+        if (!r.id.isEmpty()) {
+            const auto program = r.protocol == GpgME::OpenPGP ? gpg : gpgsm;
+            const auto headerLine = i18nc("file name (imported with gpg/gpgsm)", "%1 (imported with %2)").arg(r.id, program);
+            s += QStringLiteral("<div><b>%1</b></div>").arg(headerLine);
+        }
+        if (r.auditLog.error().code() == GPG_ERR_NO_DATA) {
+            s += QStringLiteral("<em>") + i18nc("@info", "Audit log is empty.") + QStringLiteral("</em>");
+        } else if (r.result.error().isCanceled()) {
+            s += QStringLiteral("<em>") + i18nc("@info", "Import was canceled.") + QStringLiteral("</em>");
+        } else {
+            s += r.auditLog.text();
+        }
+        return s;
+    };
+    std::transform(res.cbegin(), res.cend(), std::back_inserter(auditLogs), extractAndAnnotateAuditLog);
+    return AuditLogEntry{auditLogs.join(QLatin1String{"<hr>"}), Error{}};
+}
 }
 
 void ImportCertificatesCommand::Private::showDetails(const std::vector<ImportResultData> &res,
@@ -487,8 +519,7 @@ void ImportCertificatesCommand::Private::showDetails(const std::vector<ImportRes
         }
     }
     setImportResultProxyModel(res);
-    information(make_message_report(res, groups),
-                i18n("Certificate Import Result"));
+    MessageBox::information(parentWidgetOrView(), make_message_report(res, groups), consolidatedAuditLogEntries(res), i18n("Certificate Import Result"));
 }
 
 static QString make_error_message(const Error &err, const QString &id)
@@ -506,18 +537,9 @@ static QString make_error_message(const Error &err, const QString &id)
                   id, QString::fromLocal8Bit(err.asString()));
 }
 
-void ImportCertificatesCommand::Private::showError(QWidget *parent, const Error &err, const QString &id)
+void ImportCertificatesCommand::Private::showError(const ImportResultData &result)
 {
-    if (parent) {
-        KMessageBox::error(parent, make_error_message(err, id), i18n("Certificate Import Failed"));
-    } else {
-        showError(err, id);
-    }
-}
-
-void ImportCertificatesCommand::Private::showError(const Error &err, const QString &id)
-{
-    error(make_error_message(err, id), i18n("Certificate Import Failed"));
+    MessageBox::error(parentWidgetOrView(), make_error_message(result.result.error(), result.id), result.auditLog);
 }
 
 void ImportCertificatesCommand::Private::setWaitForMoreJobs(bool wait)
@@ -531,56 +553,83 @@ void ImportCertificatesCommand::Private::setWaitForMoreJobs(bool wait)
     }
 }
 
-void ImportCertificatesCommand::Private::importResult(const ImportResult &result, QGpgME::Job *finishedJob)
+void ImportCertificatesCommand::Private::onImportResult(const ImportResult &result, QGpgME::Job *finishedJob)
 {
     if (!finishedJob) {
         finishedJob = qobject_cast<QGpgME::Job *>(q->sender());
     }
     Q_ASSERT(finishedJob);
+    qCDebug(KLEOPATRA_LOG) << q << __func__ << finishedJob;
 
-    auto it = std::find_if(std::cbegin(jobs), std::cend(jobs),
+    auto it = std::find_if(std::begin(runningJobs), std::end(runningJobs),
                            [finishedJob](const auto &job) { return job.job == finishedJob; });
-    Q_ASSERT(it != std::cend(jobs));
-    if (it == std::cend(jobs)) {
+    Q_ASSERT(it != std::end(runningJobs));
+    if (it == std::end(runningJobs)) {
         qCWarning(KLEOPATRA_LOG) << __func__ << "Error: Finished job not found";
         return;
     }
 
-    const auto job = *it;
-    jobs.erase(std::remove(std::begin(jobs), std::end(jobs), job), std::end(jobs));
+    Kleo::for_each(it->connections, &disconnectConnection);
+    it->connections.clear();
+
     increaseProgressValue();
 
-    importResult({job.id, job.protocol, job.type, result});
+    const auto job = *it;
+    addImportResult({job.id, job.protocol, job.type, result, AuditLogEntry::fromJob(finishedJob)}, job);
 }
 
-void ImportCertificatesCommand::Private::importResult(const ImportResultData &result)
+void ImportCertificatesCommand::Private::addImportResult(const ImportResultData &result, const ImportJobData &job)
 {
-    qCDebug(KLEOPATRA_LOG) << __func__ << result.id;
+    qCDebug(KLEOPATRA_LOG) << q << __func__ << result.id << "Result:" << result.result.error().asString();
     results.push_back(result);
 
+    if (importFailed(result)) {
+        showError(result);
+    }
+
+    if (job.job) {
+        const auto count = std::erase(runningJobs, job);
+        Q_ASSERT(count == 1);
+    }
+
     tryToFinish();
 }
 
-static void handleOwnerTrust(const std::vector<ImportResultData> &results)
+static void handleOwnerTrust(const std::vector<ImportResultData> &results, QWidget *dialog)
 {
-    //iterate over all imported certificates
     for (const auto &r: results) {
-        //when a new certificate got a secret key
-        if (r.result.numSecretKeysImported() >= 1) {
-            const char *fingerPr = r.result.imports()[0].fingerprint();
-            GpgME::Error err;
-            QScopedPointer<Context>
-                ctx(Context::createForProtocol(GpgME::Protocol::OpenPGP));
+        if (r.protocol != GpgME::Protocol::OpenPGP) {
+            qCDebug(KLEOPATRA_LOG) << __func__ << "Skipping non-OpenPGP import";
+            continue;
+        }
+        const auto imports = r.result.imports();
+        for (const auto &import : imports) {
+            if (!(import.status() & (Import::Status::NewKey | Import::Status::ContainedSecretKey))) {
+                qCDebug(KLEOPATRA_LOG) << __func__ << "Skipping already known imported public key";
+                continue;
+            }
+            const char *fpr = import.fingerprint();
+            if (!fpr) {
+                qCDebug(KLEOPATRA_LOG) << __func__ << "Skipping import without fingerprint";
+                continue;
+            }
 
+            GpgME::Error err;
+            auto ctx = wrap_unique(Context::createForProtocol(GpgME::Protocol::OpenPGP));
             if (!ctx){
                 qCWarning(KLEOPATRA_LOG) << "Failed to get context";
                 continue;
             }
 
-            const Key toTrustOwner = ctx->key(fingerPr, err , false);
+            ctx->addKeyListMode(KeyListMode::WithSecret);
+            const Key toTrustOwner = ctx->key(fpr, err, false);
 
-            if (toTrustOwner.isNull()) {
-                return;
+            if (toTrustOwner.isNull() || !toTrustOwner.hasSecret()) {
+                continue;
+            }
+            if (toTrustOwner.ownerTrust() == Key::OwnerTrust::Ultimate) {
+                qCDebug(KLEOPATRA_LOG) << __func__ << "Skipping key with ultimate ownertrust";
+                continue;
             }
 
             const auto toTrustOwnerUserIDs{toTrustOwner.userIDs()};
@@ -591,33 +640,23 @@ static void handleOwnerTrust(const std::vector<ImportResultData> &results)
             });
 
             const QString str = xi18nc("@info",
-                "<title>You have imported a Secret Key.</title>"
-                "<para>The key has the fingerprint:<nl/>"
-                "<numid>%1</numid>"
-                "</para>"
-                "<para>And claims the user IDs:"
+                "<para>You have imported a certificate with fingerprint</para>"
+                "<para><numid>%1</numid></para>"
+                "<para>"
+                "and user IDs"
                 "<list>%2</list>"
                 "</para>"
-                "Is this your own key? (Set trust level to ultimate)",
-                QString::fromUtf8(fingerPr),
+                "<para>Is this your own certificate?</para>",
+                Formatting::prettyID(fpr),
                 uids);
 
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
-            int k = KMessageBox::questionTwoActions(nullptr,
-                                                    str,
-#else
-            int k = KMessageBox::questionYesNo(nullptr,
-                                               str,
-#endif
-                                                    i18nc("@title:window", "Secret key imported"),
-                                                    KGuiItem(i18n("Import")),
-                                                    KStandardGuiItem::cancel());
+            int k = KMessageBox::questionTwoActionsCancel(dialog,
+                                                          str,
+                                                          i18nc("@title:window", "Mark Own Certificate"),
+                                                          KGuiItem{i18nc("@action:button", "Yes, It's Mine")},
+                                                          KGuiItem{i18nc("@action:button", "No, It's Not Mine")});
 
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
             if (k == KMessageBox::ButtonCode::PrimaryAction) {
-#else
-            if (k == KMessageBox::Yes) {
-#endif
                 //To use the ChangeOwnerTrustJob over
                 //the CryptoBackendFactory
                 const QGpgME::Protocol *const backend = QGpgME::openpgp();
@@ -629,6 +668,9 @@ static void handleOwnerTrust(const std::vector<ImportResultData> &results)
 
                 ChangeOwnerTrustJob *const j = backend->changeOwnerTrustJob();
                 j->start(toTrustOwner, Key::Ultimate);
+            } else if (k == KMessageBox::ButtonCode::Cancel) {
+                // do not bother the user with further "Is this yours?" questions
+                return;
             }
         }
     }
@@ -665,7 +707,7 @@ void ImportCertificatesCommand::Private::processResults()
 {
     importGroups();
 
-#ifdef QGPGME_SUPPORTS_RECEIVING_KEYS_BY_KEY_ID
+#if QGPGME_SUPPORTS_RECEIVING_KEYS_BY_KEY_ID
     if (Settings{}.retrieveSignerKeysAfterImport() && !importingSignerKeys) {
         importingSignerKeys = true;
         const auto missingSignerKeys = getMissingSignerKeyIds(results);
@@ -681,7 +723,7 @@ void ImportCertificatesCommand::Private::processResults()
     // ensure that the progress dialog is closed before we show any other dialogs
     setProgressToMaximum();
 
-    handleOwnerTrust(results);
+    handleOwnerTrust(results, parentWidgetOrView());
 
     showDetails(results, importedGroups);
 
@@ -696,20 +738,42 @@ void ImportCertificatesCommand::Private::processResults()
 
 void ImportCertificatesCommand::Private::tryToFinish()
 {
-
-    if (waitForMoreJobs || !jobs.empty()) {
+    qCDebug(KLEOPATRA_LOG) << q << __func__;
+    if (waitForMoreJobs) {
+        qCDebug(KLEOPATRA_LOG) << q << __func__ << "Waiting for more jobs -> keep going";
         return;
     }
+    if (!runningJobs.empty()) {
+        qCDebug(KLEOPATRA_LOG) << q << __func__ << "There are unfinished jobs -> keep going";
+        return;
+    }
+#if QGPGME_SUPPORTS_DEFERRED_IMPORT_JOB
+    if (!pendingJobs.empty()) {
+        qCDebug(KLEOPATRA_LOG) << q << __func__ << "There are pending jobs -> start the next one";
+        auto job = pendingJobs.front();
+        pendingJobs.pop();
+        job.job->startNow();
+        runningJobs.push_back(job);
+        return;
+    }
+#endif
 
-    auto keyCache = KeyCache::mutableInstance();
-    keyListConnection = connect(keyCache.get(), &KeyCache::keyListingDone,
-                                q, [this]() { keyCacheUpdated(); });
-    keyCache->startKeyListing();
+    if (keyListConnection) {
+        qCWarning(KLEOPATRA_LOG) << q << __func__ << "There is already a valid keyListConnection!";
+    } else {
+        auto keyCache = KeyCache::mutableInstance();
+        keyListConnection = connect(keyCache.get(), &KeyCache::keyListingDone,
+                                    q, [this]() { keyCacheUpdated(); });
+        keyCache->startKeyListing();
+    }
 }
 
 void ImportCertificatesCommand::Private::keyCacheUpdated()
 {
-    disconnect(keyListConnection);
+    qCDebug(KLEOPATRA_LOG) << q << __func__;
+    if (!disconnect(keyListConnection)) {
+        qCWarning(KLEOPATRA_LOG) << q << __func__ << "Failed to disconnect keyListConnection";
+    }
 
     keyCacheAutoRefreshSuspension.reset();
 
@@ -739,19 +803,6 @@ void ImportCertificatesCommand::Private::keyCacheUpdated()
         return;
     }
 
-    if (std::any_of(std::cbegin(results), std::cend(results), &importFailed)) {
-        // ensure that the progress dialog is closed before we show any other dialogs
-        setProgressToMaximum();
-        setImportResultProxyModel(results);
-        for (const auto &r : results) {
-            if (importFailed(r)) {
-                showError(r.result.error(), r.id);
-            }
-        }
-        finished();
-        return;
-    }
-
     processResults();
 }
 
@@ -865,30 +916,43 @@ void ImportCertificatesCommand::Private::startImport(GpgME::Protocol protocol, c
         error(i18n("The type of this certificate (%1) is not supported by this Kleopatra installation.",
                    Formatting::displayName(protocol)),
               i18n("Certificate Import Failed"));
-        importResult({id, protocol, ImportType::Local, ImportResult{}});
+        addImportResult({id, protocol, ImportType::Local, ImportResult{}, AuditLogEntry{}});
         return;
     }
 
     keyCacheAutoRefreshSuspension = KeyCache::mutableInstance()->suspendAutoRefresh();
 
     std::vector<QMetaObject::Connection> connections = {
-        connect(job.get(), &AbstractImportJob::result, q, [this](const GpgME::ImportResult &result) { importResult(result); }),
-        connect(job.get(), &Job::progress,
-                q, &Command::progress)
+        connect(job.get(), &AbstractImportJob::result, q, [this](const GpgME::ImportResult &result) { onImportResult(result); }),
+#if QGPGME_JOB_HAS_NEW_PROGRESS_SIGNALS
+        connect(job.get(), &QGpgME::Job::jobProgress,
+                q, &Command::progress),
+#else
+        connect(job.get(), &QGpgME::Job::progress,
+                q, [this](const QString &, int current, int total) { Q_EMIT q->progress(current, total); }),
+#endif
     };
 
-#ifdef QGPGME_SUPPORTS_IMPORT_WITH_FILTER
+#if QGPGME_SUPPORTS_IMPORT_WITH_FILTER
     job->setImportFilter(options.importFilter);
 #endif
-#ifdef QGPGME_SUPPORTS_IMPORT_WITH_KEY_ORIGIN
+#if QGPGME_SUPPORTS_IMPORT_WITH_KEY_ORIGIN
     job->setKeyOrigin(options.keyOrigin, options.keyOriginUrl);
 #endif
+#if QGPGME_SUPPORTS_DEFERRED_IMPORT_JOB
+    const GpgME::Error err = job->startLater(data);
+#else
     const GpgME::Error err = job->start(data);
+#endif
     if (err.code()) {
-        importResult({id, protocol, ImportType::Local, ImportResult{err}});
+        addImportResult({id, protocol, ImportType::Local, ImportResult{err}, AuditLogEntry{}});
     } else {
         increaseProgressMaximum();
-        jobs.push_back({id, protocol, ImportType::Local, job.release(), connections});
+#if QGPGME_SUPPORTS_DEFERRED_IMPORT_JOB
+        pendingJobs.push({id, protocol, ImportType::Local, job.release(), connections});
+#else
+        runningJobs.push_back({id, protocol, ImportType::Local, job.release(), connections});
+#endif
     }
 }
 
@@ -916,24 +980,29 @@ void ImportCertificatesCommand::Private::startImport(GpgME::Protocol protocol, c
         error(i18n("The type of this certificate (%1) is not supported by this Kleopatra installation.",
                    Formatting::displayName(protocol)),
               i18n("Certificate Import Failed"));
-        importResult({id, protocol, ImportType::External, ImportResult{}});
+        addImportResult({id, protocol, ImportType::External, ImportResult{}, AuditLogEntry{}});
         return;
     }
 
     keyCacheAutoRefreshSuspension = KeyCache::mutableInstance()->suspendAutoRefresh();
 
     std::vector<QMetaObject::Connection> connections = {
-        connect(job.get(), &AbstractImportJob::result, q, [this](const GpgME::ImportResult &result) { importResult(result); }),
-        connect(job.get(), &Job::progress,
-                q, &Command::progress)
+        connect(job.get(), &AbstractImportJob::result, q, [this](const GpgME::ImportResult &result) { onImportResult(result); }),
+#if QGPGME_JOB_HAS_NEW_PROGRESS_SIGNALS
+        connect(job.get(), &QGpgME::Job::jobProgress,
+                q, &Command::progress),
+#else
+        connect(job.get(), &QGpgME::Job::progress,
+                q, [this](const QString &, int current, int total) { Q_EMIT q->progress(current, total); }),
+#endif
     };
 
     const GpgME::Error err = job->start(keys);
     if (err.code()) {
-        importResult({id, protocol, ImportType::External, ImportResult{err}});
+        addImportResult({id, protocol, ImportType::External, ImportResult{err}, AuditLogEntry{}});
     } else {
         increaseProgressMaximum();
-        jobs.push_back({id, protocol, ImportType::External, job.release(), connections});
+        runningJobs.push_back({id, protocol, ImportType::External, job.release(), connections});
     }
 }
 
@@ -941,7 +1010,7 @@ static auto get_receive_keys_job(GpgME::Protocol protocol)
 {
     Q_ASSERT(protocol != UnknownProtocol);
 
-#ifdef QGPGME_SUPPORTS_RECEIVING_KEYS_BY_KEY_ID
+#if QGPGME_SUPPORTS_RECEIVING_KEYS_BY_KEY_ID
     std::unique_ptr<ReceiveKeysJob> job{};
     if (const auto backend = (protocol == GpgME::OpenPGP ? QGpgME::openpgp() : QGpgME::smime())) {
         job.reset(backend->receiveKeysJob());
@@ -959,27 +1028,32 @@ void ImportCertificatesCommand::Private::startImport(GpgME::Protocol protocol, [
     auto job = get_receive_keys_job(protocol);
     if (!job.get()) {
         qCWarning(KLEOPATRA_LOG) << "Failed to get ReceiveKeysJob for protocol" << Formatting::displayName(protocol);
-        importResult({id, protocol, ImportType::External, ImportResult{}});
+        addImportResult({id, protocol, ImportType::External, ImportResult{}, AuditLogEntry{}});
         return;
     }
 
-#ifdef QGPGME_SUPPORTS_RECEIVING_KEYS_BY_KEY_ID
+#if QGPGME_SUPPORTS_RECEIVING_KEYS_BY_KEY_ID
     keyCacheAutoRefreshSuspension = KeyCache::mutableInstance()->suspendAutoRefresh();
 
     std::vector<QMetaObject::Connection> connections = {
         connect(job.get(), &AbstractImportJob::result, q, [this](const GpgME::ImportResult &result) {
-            importResult(result);
+            onImportResult(result);
         }),
-        connect(job.get(), &Job::progress,
-                q, &Command::progress)
+#if QGPGME_JOB_HAS_NEW_PROGRESS_SIGNALS
+        connect(job.get(), &QGpgME::Job::jobProgress,
+                q, &Command::progress),
+#else
+        connect(job.get(), &QGpgME::Job::progress,
+                q, [this](const QString &, int current, int total) { Q_EMIT q->progress(current, total); }),
+#endif
     };
 
     const GpgME::Error err = job->start(keyIds);
     if (err.code()) {
-        importResult({id, protocol, ImportType::External, ImportResult{err}});
+        addImportResult({id, protocol, ImportType::External, ImportResult{err}, AuditLogEntry{}});
     } else {
         increaseProgressMaximum();
-        jobs.push_back({id, protocol, ImportType::External, job.release(), connections});
+        runningJobs.push_back({id, protocol, ImportType::External, job.release(), connections});
     }
 #endif
 }
@@ -996,7 +1070,9 @@ void ImportCertificatesCommand::Private::setUpProgressDialog()
         return;
     }
     progressDialog = new QProgressDialog{parentWidgetOrView()};
-    progressDialog->setModal(true);
+    // use a non-modal progress dialog to avoid reentrancy problems (and crashes) if multiple jobs finish in the same event loop cycle
+    // (cf. the warning for QProgressDialog::setValue() in the API documentation)
+    progressDialog->setModal(false);
     progressDialog->setWindowTitle(progressWindowTitle);
     progressDialog->setLabelText(progressLabelText);
     progressDialog->setMinimumDuration(1000);
@@ -1043,25 +1119,17 @@ void ImportCertificatesCommand::Private::setProgressToMaximum()
     progressDialog->setValue(progressDialog->maximum());
 }
 
-static void disconnectConnection(const QMetaObject::Connection &connection)
-{
-    // trivial function for disconnecting a signal-slot connection because
-    // using a lambda seems to confuse older GCC / MinGW and unnecessarily
-    // capturing 'this' generates warnings
-    QObject::disconnect(connection);
-}
-
 void ImportCertificatesCommand::doCancel()
 {
-    const auto jobsToCancel = d->jobs;
-    std::for_each(std::begin(jobsToCancel), std::end(jobsToCancel),
-                  [this](const auto &job) {
-                      qCDebug(KLEOPATRA_LOG) << "Canceling job" << job.job;
-                      std::for_each(std::cbegin(job.connections), std::cend(job.connections),
-                                    &disconnectConnection);
-                      job.job->slotCancel();
-                      d->importResult(ImportResult{Error::fromCode(GPG_ERR_CANCELED)}, job.job);
-                  });
+    const auto jobsToCancel = d->runningJobs;
+    std::for_each(std::begin(jobsToCancel), std::end(jobsToCancel), [this](const auto &job) {
+        if (!job.connections.empty()) {
+            // ignore jobs without connections; they are already completed
+            qCDebug(KLEOPATRA_LOG) << "Canceling job" << job.job;
+            job.job->slotCancel();
+            d->onImportResult(ImportResult{Error::fromCode(GPG_ERR_CANCELED)}, job.job);
+        }
+    });
 }
 
 #undef d
