@@ -14,11 +14,15 @@
 #include "command_p.h"
 #include "importcertificatescommand.h"
 
+#include <Libkleo/AuditLogEntry>
 #include <Libkleo/KeyGroup>
 
 #include <gpgme++/global.h>
 #include <gpgme++/importresult.h>
 
+#if QGPGME_SUPPORTS_DEFERRED_IMPORT_JOB
+#include <queue>
+#endif
 #include <vector>
 #include <map>
 
@@ -44,6 +48,7 @@ class QProgressDialog;
 
 enum class ImportType
 {
+    Unknown,
     Local,
     External,
 };
@@ -51,9 +56,9 @@ enum class ImportType
 struct ImportJobData
 {
     QString id;
-    GpgME::Protocol protocol;
-    ImportType type;
-    QGpgME::Job *job;
+    GpgME::Protocol protocol = GpgME::UnknownProtocol;
+    ImportType type = ImportType::Unknown;
+    QGpgME::Job *job = nullptr;
     std::vector<QMetaObject::Connection> connections;
 };
 
@@ -62,9 +67,10 @@ bool operator==(const ImportJobData &lhs, const ImportJobData &rhs);
 struct ImportResultData
 {
     QString id;
-    GpgME::Protocol protocol;
-    ImportType type;
+    GpgME::Protocol protocol = GpgME::UnknownProtocol;
+    ImportType type = ImportType::Unknown;
     GpgME::ImportResult result;
+    Kleo::AuditLogEntry auditLog;
 };
 
 struct ImportedGroup
@@ -105,13 +111,12 @@ public:
     void startImport(GpgME::Protocol proto, const QByteArray &data, const QString &id = QString(), const ImportOptions &options = {});
     void startImport(GpgME::Protocol proto, const std::vector<GpgME::Key> &keys, const QString &id = QString());
     void startImport(GpgME::Protocol proto, const QStringList &keyIds, const QString &id = {});
-    void importResult(const GpgME::ImportResult &, QGpgME::Job *job = nullptr);
-    void importResult(const ImportResultData &result);
+    void onImportResult(const GpgME::ImportResult &, QGpgME::Job *job = nullptr);
+    void addImportResult(const ImportResultData &result, const ImportJobData &job = ImportJobData{});
 
     void importGroupsFromFile(const QString &filename);
 
-    void showError(QWidget *parent, const GpgME::Error &error, const QString &id = QString());
-    void showError(const GpgME::Error &error, const QString &id = QString());
+    void showError(const ImportResultData &result);
 
     void setImportResultProxyModel(const std::vector<ImportResultData> &results);
 
@@ -140,7 +145,10 @@ private:
     bool waitForMoreJobs = false;
     bool importingSignerKeys = false;
     std::vector<GpgME::Protocol> nonWorkingProtocols;
-    std::vector<ImportJobData> jobs;
+#if QGPGME_SUPPORTS_DEFERRED_IMPORT_JOB
+    std::queue<ImportJobData> pendingJobs;
+#endif
+    std::vector<ImportJobData> runningJobs;
     std::vector<QString> filesToImportGroupsFrom;
     std::vector<ImportResultData> results;
     std::vector<ImportedGroup> importedGroups;
