@@ -35,7 +35,6 @@
 #include <KLocalizedString>
 
 #include "kleopatra_debug.h"
-#include <kwidgetsaddons_version.h>
 
 KleoPageConfigDialog::KleoPageConfigDialog(QWidget *parent)
     : KPageDialog(parent)
@@ -87,11 +86,7 @@ void KleoPageConfigDialog::slotCurrentPageChanged(KPageWidgetItem *current, KPag
     KCModule *previousModule = qobject_cast<KCModule*>(previous->widget());
     bool canceled = false;
     if (previousModule && mChangedModules.contains(previousModule)) {
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
         const int queryUser = KMessageBox::warningTwoActionsCancel(
-#else
-        const int queryUser = KMessageBox::warningYesNoCancel(
-#endif
             this,
             i18n("The settings of the current module have changed.\n"
                  "Do you want to apply the changes or discard them?"),
@@ -99,17 +94,9 @@ void KleoPageConfigDialog::slotCurrentPageChanged(KPageWidgetItem *current, KPag
             KStandardGuiItem::apply(),
             KStandardGuiItem::discard(),
             KStandardGuiItem::cancel());
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
         if (queryUser == KMessageBox::ButtonCode::PrimaryAction) {
-#else
-        if (queryUser == KMessageBox::Yes) {
-#endif
             previousModule->save();
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
         } else if (queryUser == KMessageBox::ButtonCode::SecondaryAction) {
-#else
-        } else if (queryUser == KMessageBox::No) {
-#endif
             previousModule->load();
         }
         canceled = queryUser == KMessageBox::Cancel;
@@ -194,14 +181,8 @@ void KleoPageConfigDialog::slotHelpClicked()
     docUrl = QUrl(QStringLiteral("help:/")).resolved(QUrl(docPath)); // same code as in KHelpClient::invokeHelp
 #endif
     if (docUrl.scheme() == QLatin1String("help") || docUrl.scheme() == QLatin1String("man") || docUrl.scheme() == QLatin1String("info")) {
-      const QString exec =
-          QStandardPaths::findExecutable(QStringLiteral("khelpcenter"));
-      if (exec.isEmpty()) {
-        qCWarning(KLEOPATRA_LOG) << "Could not find khelpcenter in PATH.";
-      } else {
-        QProcess::startDetached(QStringLiteral("khelpcenter"),
-                                QStringList() << docUrl.toString());
-      }
+        // Warning: Don't assume that the program needs to be in PATH. On Windows, it will also be found next to the calling process.
+        QProcess::startDetached(QStringLiteral("khelpcenter"), QStringList() << docUrl.toString());
     } else {
         QDesktopServices::openUrl(docUrl);
     }
