@@ -35,7 +35,10 @@
 #include <KLocalizedString>
 #include <KMessageBox>
 #include "kleopatra_debug.h"
-#include <kwidgetsaddons_version.h>
+
+#if QGPGME_SUPPORTS_ARCHIVE_JOBS
+#include <QGpgME/DecryptVerifyArchiveJob>
+#endif
 
 #include <QDir>
 #include <QFile>
@@ -180,7 +183,7 @@ void AutoDecryptVerifyFilesController::Private::exec()
                 // A directory. Assume that the input was an archive
                 // and avoid directory merges by trying to find a non
                 // existing directory.
-                auto candidate = fi.baseName();
+                auto candidate = fi.fileName();
                 if (candidate.startsWith(QLatin1Char('-'))) {
                     // Bug in GpgTar Extracts stdout passed archives to a dir named -
                     candidate = QFileInfo(m_passedFiles.first()).baseName();
@@ -209,13 +212,8 @@ void AutoDecryptVerifyFilesController::Private::exec()
             QString outFileName = fi.fileName();
             if (!embeddedFileName.isEmpty() && embeddedFileName != fi.fileName()) {
                 // we switch "Yes" and "No" because Yes is default, but saving with embedded file name could be dangerous
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
                 const auto answer = KMessageBox::questionTwoActionsCancel(
                     m_dialog,
-#else
-                const auto answer =
-                    KMessageBox::questionYesNoCancel(m_dialog,
-#endif
                     xi18n("Shall the file be saved with the original file name <filename>%1</filename>?", embeddedFileName),
                     i18n("Use Original File Name?"),
                     KGuiItem(xi18n("No, Save As <filename>%1</filename>", fi.fileName())),
@@ -223,11 +221,7 @@ void AutoDecryptVerifyFilesController::Private::exec()
                 if (answer == KMessageBox::Cancel) {
                     qCDebug(KLEOPATRA_LOG) << "Saving canceled for:" << inpath;
                     continue;
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
                 } else if (answer == KMessageBox::ButtonCode::SecondaryAction) {
-#else
-                } else if (answer == KMessageBox::No) {
-#endif
                     outFileName = embeddedFileName;
                 }
             }
@@ -237,13 +231,8 @@ void AutoDecryptVerifyFilesController::Private::exec()
             if (ofi.exists()) {
                 int sel = KMessageBox::Cancel;
                 if (!overWriteAll) {
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
                     sel = KMessageBox::questionTwoActionsCancel(m_dialog,
                                                                 i18n("The file <b>%1</b> already exists.\n"
-#else
-                    sel = KMessageBox::questionYesNoCancel(m_dialog,
-                                                           i18n("The file <b>%1</b> already exists.\n"
-#endif
                                                                      "Overwrite?",
                                                                      outpath),
                                                                 i18n("Overwrite Existing File?"),
@@ -255,11 +244,7 @@ void AutoDecryptVerifyFilesController::Private::exec()
                     qCDebug(KLEOPATRA_LOG) << "Overwriting canceled for: " << outpath;
                     continue;
                 }
-#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
                 if (sel == KMessageBox::ButtonCode::SecondaryAction) { // Overwrite All
-#else
-                if (sel == KMessageBox::No) { //Overwrite All
-#endif
                     overWriteAll = true;
                 }
                 if (!QFile::remove(outpath)) {
@@ -323,6 +308,14 @@ QVector<AutoDecryptVerifyFilesController::Private::CryptoFile> AutoDecryptVerify
     return out;
 }
 
+static bool archiveJobsCanBeUsed([[maybe_unused]] GpgME::Protocol protocol)
+{
+#if QGPGME_SUPPORTS_ARCHIVE_JOBS
+    return (protocol == GpgME::OpenPGP) && QGpgME::DecryptVerifyArchiveJob::isSupported();
+#else
+    return false;
+#endif
+}
 
 std::vector< std::shared_ptr<Task> > AutoDecryptVerifyFilesController::Private::buildTasks(const QStringList &fileNames, QStringList &undetected)
 {
@@ -464,8 +457,16 @@ std::vector< std::shared_ptr<Task> > AutoDecryptVerifyFilesController::Private::
 
             const auto wd = QDir(m_workDir->path());
 
-            const auto output = ad ? ad->createOutputFromUnpackCommand(cFile.protocol, cFile.fileName, wd)
-                                   : Output::createFromFile(wd.absoluteFilePath(outputFileName(fi.fileName())), false);
+            std::shared_ptr<Output> output;
+            if (ad) {
+                if ((ad->id() == QLatin1String{"tar"}) && archiveJobsCanBeUsed(cFile.protocol)) {
+                    // we don't need an output
+                } else {
+                    output = ad->createOutputFromUnpackCommand(cFile.protocol, cFile.fileName, wd);
+                }
+            } else {
+                output = Output::createFromFile(wd.absoluteFilePath(outputFileName(fi.fileName())), false);
+            }
 
             // If this might be opaque CMS signature, then try that. We already handled
             // detached CMS signature above
@@ -475,8 +476,14 @@ std::vector< std::shared_ptr<Task> > AutoDecryptVerifyFilesController::Private::
                 qCDebug(KLEOPATRA_LOG) << "creating a VerifyOpaqueTask";
                 std::shared_ptr<VerifyOpaqueTask> t(new VerifyOpaqueTask);
                 t->setInput(input);
-                t->setOutput(output);
+                if (output) {
+                    t->setOutput(output);
+                }
                 t->setProtocol(cFile.protocol);
+                if (ad) {
+                    t->setExtractArchive(true);
+                    t->setOutputDirectory(m_workDir->path());
+                }
                 tasks.push_back(t);
             } else {
                 // Any message. That is not an opaque signature needs to be
@@ -485,8 +492,14 @@ std::vector< std::shared_ptr<Task> > AutoDecryptVerifyFilesController::Private::
                 qCDebug(KLEOPATRA_LOG) << "creating a DecryptVerifyTask";
                 std::shared_ptr<DecryptVerifyTask> t(new DecryptVerifyTask);
                 t->setInput(input);
-                t->setOutput(output);
+                if (output) {
+                    t->setOutput(output);
+                }
                 t->setProtocol(cFile.protocol);
+                if (ad) {
+                    t->setExtractArchive(true);
+                    t->setOutputDirectory(m_workDir->path());
+                }
                 cFile.output = output;
                 tasks.push_back(t);
             }
