@@ -6,8 +6,7 @@
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
-#ifndef DIALOGS_GENCARDKEYDIALOG_H
-#define DIALOGS_GENCARDKEYDIALOG_H
+#pragma once
 
 #include <QDialog>
 
@@ -18,6 +17,11 @@
 
 namespace Kleo
 {
+namespace SmartCard
+{
+struct AlgorithmInfo;
+}
+
 class GenCardKeyDialog: public QDialog
 {
 Q_OBJECT
@@ -50,7 +54,7 @@ public:
 
     KeyParams getKeyParams() const;
 
-    void setSupportedAlgorithms(const std::vector<std::pair<std::string, QString>> &algorithms, const std::string &defaultAlgo);
+    void setSupportedAlgorithms(const std::vector<SmartCard::AlgorithmInfo> &algorithms, const std::string &defaultAlgo);
 
 private:
     class Private;
@@ -59,4 +63,3 @@ private:
 } // namespace Kleo
 
 
-#endif // DIALOGS_GENCARDKEYDIALOG_H
