@@ -17,6 +17,8 @@
 #ifdef Q_OS_WIN // HACK: AllowSetForegroundWindow needs _WIN32_WINDOWS >= 0x0490 set
 # ifndef _WIN32_WINDOWS
 #  define _WIN32_WINDOWS 0x0500
+# endif
+# ifndef _WIN32_WINNT
 #  define _WIN32_WINNT 0x0500 // good enough for Vista too
 # endif
 # include <utils/gnupg-registry.h>
@@ -34,11 +36,10 @@
 #include <gpg-error.h>
 #include <gpgme++/global.h>
 
-#include <QStandardPaths>
 #include <algorithm>
-#include <memory>
-#include <sstream>
 #include <string>
+#include <sstream>
+#include <memory>
 #include <type_traits>
 
 using namespace KleopatraClientCopy;
@@ -369,49 +370,34 @@ QByteArray Command::command() const
 // here comes the ugly part
 //
 
-#ifdef HAVE_ASSUAN2
 static void my_assuan_release(assuan_context_t ctx)
 {
     if (ctx) {
         assuan_release(ctx);
     }
 }
-#endif
 
 using AssuanContextBase = std::shared_ptr<std::remove_pointer<assuan_context_t>::type>;
 namespace
 {
 struct AssuanClientContext : AssuanContextBase {
     AssuanClientContext() : AssuanContextBase() {}
-#ifndef HAVE_ASSUAN2
-    explicit AssuanClientContext(assuan_context_t ctx) : AssuanContextBase(ctx, &assuan_disconnect) {}
-    void reset(assuan_context_t ctx = nullptr)
-    {
-        AssuanContextBase::reset(ctx, &assuan_disconnect);
-    }
-#else
     explicit AssuanClientContext(assuan_context_t ctx) : AssuanContextBase(ctx, &my_assuan_release) {}
     void reset(assuan_context_t ctx = nullptr)
     {
         AssuanContextBase::reset(ctx, &my_assuan_release);
     }
-#endif
 };
 }
 
-#ifdef HAVE_ASSUAN2
-// compatibility typedef - remove when we require assuan v2...
-using assuan_error_t = gpg_error_t;
-#endif
-
-static assuan_error_t
+static gpg_error_t
 my_assuan_transact(const AssuanClientContext &ctx,
                    const char *command,
-                   assuan_error_t (*data_cb)(void *, const void *, size_t) = nullptr,
+                   gpg_error_t (*data_cb)(void *, const void *, size_t) = nullptr,
                    void *data_cb_arg = nullptr,
-                   assuan_error_t (*inquire_cb)(void *, const char *) = nullptr,
+                   gpg_error_t (*inquire_cb)(void *, const char *) = nullptr,
                    void *inquire_cb_arg = nullptr,
-                   assuan_error_t (*status_cb)(void *, const char *) = nullptr,
+                   gpg_error_t (*status_cb)(void *, const char *) = nullptr,
                    void *status_cb_arg = nullptr)
 {
     return assuan_transact(ctx.get(), command, data_cb, data_cb_arg, inquire_cb, inquire_cb_arg, status_cb, status_cb_arg);
@@ -459,27 +445,22 @@ static QString uiserver_executable()
 
 static QString start_uiserver()
 {
-  const QString executable = uiserver_executable();
-  const QString exec = QStandardPaths::findExecutable(executable);
-  if (exec.isEmpty()) {
-    qCWarning(LIBKLEOPATRACLIENTCORE_LOG)
-        << "Could not find " << executable << " in PATH.";
-    return i18n("Failed to start uiserver %1", executable);
-  } else {
-    QProcess::startDetached(executable, QStringList()
-                                            << QStringLiteral("--daemon"));
-  }
-  return QString();
+    // Warning: Don't assume that the program needs to be in PATH. On Windows, it will also be found next to the calling process.
+    if (!QProcess::startDetached(uiserver_executable(), QStringList() << QStringLiteral("--daemon"))) {
+        return i18n("Failed to start uiserver %1", uiserver_executable());
+    } else {
+        return QString();
+    }
 }
 
-static assuan_error_t getinfo_pid_cb(void *opaque, const void *buffer, size_t length)
+static gpg_error_t getinfo_pid_cb(void *opaque, const void *buffer, size_t length)
 {
     qint64 &pid = *static_cast<qint64 *>(opaque);
     pid = QByteArray(static_cast<const char *>(buffer), length).toLongLong();
     return 0;
 }
 
-static assuan_error_t command_data_cb(void *opaque, const void *buffer, size_t length)
+static gpg_error_t command_data_cb(void *opaque, const void *buffer, size_t length)
 {
     QByteArray &ba = *static_cast<QByteArray *>(opaque);
     ba.append(QByteArray(static_cast<const char *>(buffer), length));
@@ -494,7 +475,7 @@ struct inquire_data {
 };
 }
 
-static assuan_error_t command_inquire_cb(void *opaque, const char *what)
+static gpg_error_t command_inquire_cb(void *opaque, const char *what)
 {
     if (!opaque) {
         return 0;
@@ -513,7 +494,7 @@ static inline std::ostream &operator<<(std::ostream &s, const QByteArray &ba)
     return s << std::string(ba.data(), ba.size());
 }
 
-static assuan_error_t send_option(const AssuanClientContext &ctx, const char *name, const QVariant &value)
+static gpg_error_t send_option(const AssuanClientContext &ctx, const char *name, const QVariant &value)
 {
     std::stringstream ss;
     ss << "OPTION " << name;
@@ -523,14 +504,14 @@ static assuan_error_t send_option(const AssuanClientContext &ctx, const char *na
     return my_assuan_transact(ctx, ss.str().c_str());
 }
 
-static assuan_error_t send_file(const AssuanClientContext &ctx, const QString &file)
+static gpg_error_t send_file(const AssuanClientContext &ctx, const QString &file)
 {
     std::stringstream ss;
     ss << "FILE " << hexencode(QFile::encodeName(file));
     return my_assuan_transact(ctx, ss.str().c_str());
 }
 
-static assuan_error_t send_recipient(const AssuanClientContext &ctx, const QString &recipient, bool info)
+static gpg_error_t send_recipient(const AssuanClientContext &ctx, const QString &recipient, bool info)
 {
     std::stringstream ss;
     ss << "RECIPIENT ";
@@ -541,7 +522,7 @@ static assuan_error_t send_recipient(const AssuanClientContext &ctx, const QStri
     return my_assuan_transact(ctx, ss.str().c_str());
 }
 
-static assuan_error_t send_sender(const AssuanClientContext &ctx, const QString &sender, bool info)
+static gpg_error_t send_sender(const AssuanClientContext &ctx, const QString &sender, bool info)
 {
     std::stringstream ss;
     ss << "SENDER ";
@@ -570,11 +551,8 @@ void Command::Private::run()
         out.serverLocation = default_socket_name();
     }
 
-#ifndef HAVE_ASSUAN2
-    assuan_context_t naked_ctx = 0;
-#endif
     AssuanClientContext ctx;
-    assuan_error_t err = 0;
+    gpg_error_t err = 0;
 
     inquire_data id = { &in.inquireData, &ctx };
 
@@ -584,9 +562,6 @@ void Command::Private::run()
         goto leave;
     }
 
-#ifndef HAVE_ASSUAN2
-    err = assuan_socket_connect(&naked_ctx, socketName.toUtf8().constData(), -1);
-#else
     {
         assuan_context_t naked_ctx = nullptr;
         err = assuan_new(&naked_ctx);
@@ -600,7 +575,6 @@ void Command::Private::run()
     }
 
     err = assuan_socket_connect(ctx.get(), socketName.toUtf8().constData(), -1, 0);
-#endif
     if (err) {
         qDebug("UI server not running, starting it");
 
@@ -623,11 +597,6 @@ void Command::Private::run()
         goto leave;
     }
 
-#ifndef HAVE_ASSUAN2
-    ctx.reset(naked_ctx);
-    naked_ctx = 0;
-#endif
-
     out.serverPid = -1;
     err = my_assuan_transact(ctx, "GETINFO pid", &getinfo_pid_cb, &out.serverPid);
     if (err || out.serverPid <= 0) {
@@ -648,11 +617,7 @@ void Command::Private::run()
     }
 
     if (in.parentWId) {
-#if defined(Q_OS_WIN32)
-        err = send_option(ctx, "window-id", QString::asprintf("%lx", reinterpret_cast<quintptr>(in.parentWId)));
-#else
-        err = send_option(ctx, "window-id", QString::asprintf("%lx", static_cast<unsigned long>(in.parentWId)));
-#endif
+        err = send_option(ctx, "window-id", QString::number(in.parentWId, 16));
         if (err) {
             qDebug("sending option window-id failed - ignoring");
         }
